private class TranslateTask : GLib.Object {
    private bool isActive;
    private string from;
    private string to;
    private string text;
    private TranslateService service;
    
    private Json.Parser parser = new Json.Parser ();        

    public void run() {
        Thread.usleep(300000);
        try {
            if (!isActive) return;
            
            var session = new Soup.SessionSync ();
            session.timeout = GlobalSettings.SERVER_RESPOND_TIMEOUT;
            session.proxy_uri = GlobalSettings.getProxyUri();
            
            var ntext = Soup.URI.encode(text, null);
            var etext = @"http://translate.google.ru/translate_a/t?client=x&hl=$(from)&sl=$(from)&tl=$(to)&oe=UTF-8&ie=UTF-8&q=$(ntext)";
            var url = new Soup.URI(etext);
            var message = new Soup.Message.from_uri ("GET", url);
            if (!isActive) return;
            var status = session.send_message (message);        
            var mess = (string)message.response_body.data;            
            
            var data = new Gee.ArrayList<string>();
            parser.load_from_data ((string)mess);
            if (!isActive) return;            
            var root = parser.get_root ().get_object ();
            if (root != null) {
                var sentences = root.get_array_member("sentences");
                if (sentences != null) {                
                    foreach (var s in sentences.get_elements()) {
                        var el = s.get_object();
                        if (el == null) continue;
                        var trans = el.get_string_member("trans");
                        data.add(trans);
                    }
                }
                service.onData(data.to_array());
            }
            
            if (!isActive) return;
            isActive = false;
        } catch (Error e) {
            stderr.printf(e.message);
        }
    }

    public TranslateTask(TranslateService service,string from, string to, string text) {
        this.service = service;
        this.from = from;
        this.to = to;
        this.text = text;    
    }
    
    public void start() {
        isActive = true;
        run();
    }
    
    public void stop() {
        isActive = false;
    }
}

public class TranslateService : GLib.Object {    
    
    private ThreadPool<TranslateTask> pool;  
    private TranslateTask task;
    private string[] res;
    
    public signal void result(string[] text);

    public TranslateService() {
        pool = new ThreadPool<TranslateTask>.with_owned_data ((worker) => {			
			worker.start ();
		}, 7, false);
    }

    public void translate(string from, string to, string text) {
        try { 
            if (task != null) {
                task.stop();
            }       
            task = new TranslateTask(this, from, to, text);
            pool.add(task);
        } catch {
        }
    }
    
    public void onData(string[] text) {
        res = text;    
        Timeout.add (1, () => {
            result(res);
            return false;
        });
    }
}