public class TranslateWindow : Gtk.ApplicationWindow {
    private TranslateService service;

    private Gtk.HeaderBar header;
    private Gtk.Button changeButton;
    private Gtk.TextView topText;
    private Gtk.TextView bottomText;
    private Gtk.ComboBox leftLangCombo;
    private Gtk.ComboBox rightLangCombo;
    private Gtk.ListStore langStore;
    
    private static int DEFAULT_WIDTH = 0;
    private static int DEFAULT_HEIGHT = 640;
    
    private LangInfo[] langs;
        
    private string leftLang;
    private string rightLang;
    
    public TranslateWindow() {
        langs = GlobalSettings.getLangs();
    
        service = new TranslateService();
        service.result.connect(onTranslate);
    
        this.window_position = Gtk.WindowPosition.CENTER;
        this.set_gravity(Gdk.Gravity.CENTER);
        this.set_resizable(false);
        
        Gdk.RGBA bgColor = Gdk.RGBA();
        bgColor.red = 1;
        bgColor.green = 1;
        bgColor.blue = 1;
        bgColor.alpha = 1;        
        
        header = new Gtk.HeaderBar ();
        header.set_show_close_button (true);                                
				
        leftLangCombo = new Gtk.ComboBox();
        var renderer = new Gtk.CellRendererText ();
		leftLangCombo.pack_start (renderer, true);
		leftLangCombo.add_attribute (renderer, "text", 0);
		leftLangCombo.active = 0;
				
		
		rightLangCombo = new Gtk.ComboBox(); 
        rightLangCombo.set_margin_right(10);
		rightLangCombo.pack_start (renderer, true);
		rightLangCombo.add_attribute (renderer, "text", 0);
		rightLangCombo.active = 0;				
        
        changeButton = new Gtk.Button();        
        changeButton.set_image(Assets.getImage("images/loop.svg"));
        changeButton.set_margin_left(20);        
        changeButton.clicked.connect(onSwap);
        
        header.pack_start(leftLangCombo);
        header.pack_end(rightLangCombo);
        header.set_custom_title(changeButton);
        
        this.set_titlebar (header);
        this.set_size_request (DEFAULT_WIDTH, DEFAULT_HEIGHT); 
        rightLangCombo.set_size_request (110, 30);
        leftLangCombo.set_size_request (110, 30);
        
        var paned = new Gtk.Paned(Gtk.Orientation.VERTICAL);
        paned.override_background_color(Gtk.StateFlags.NORMAL, bgColor);
        var fd = new Pango.FontDescription();
        fd.set_size(11000);
        
        topText = new Gtk.TextView();        
        topText.set_margin_left(7);
        topText.set_margin_top(7);
        topText.set_margin_right(7);
        topText.override_font(fd);
        topText.set_wrap_mode(Gtk.WrapMode.WORD);
        topText.buffer.changed.connect(onUpdate);
        var topScroll = new Gtk.ScrolledWindow (null, null);       
        topScroll.set_policy (Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC);
        topScroll.set_size_request(100, 30);
        topScroll.add (topText);
        
        bottomText = new Gtk.TextView();
        bottomText.set_editable(false);
        bottomText.set_margin_top(7);
        bottomText.set_margin_right(7);
        bottomText.set_margin_left(7);
        bottomText.override_font(fd);
        bottomText.set_cursor_visible(false);
        bottomText.set_wrap_mode(Gtk.WrapMode.WORD);
        
        var bottomScroll = new Gtk.ScrolledWindow (null, null);       
        bottomScroll.set_policy (Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC);
        bottomScroll.add (bottomText);
        
        paned.pack1(topScroll, true, true);
        paned.pack2(bottomScroll, true, true);
        
        this.add(paned);                 
        
        populateLangs();        
    }
    
    private void populateLangs() {
        langStore = new Gtk.ListStore (2, typeof (string), typeof (string));
		Gtk.TreeIter iter;
        
        foreach (var l in langs) {
            langStore.append (out iter);
            langStore.@set (iter, 0, l.name, 1, l.id);
        }
        
        leftLangCombo.set_model(langStore);
        rightLangCombo.set_model(langStore);                
        
        leftLangCombo.set_id_column(1);
        rightLangCombo.set_id_column(1);
        
        leftLangCombo.active = 0;
        rightLangCombo.active = 1;
        
        leftLangCombo.changed.connect(onLeftComboChange);
		rightLangCombo.changed.connect(onRightComboChange);
				
        leftLang = getLeftId();        
        rightLang = getRightId();            
    }
    
    private string getLeftId() {
        Value val;
        Gtk.TreeIter iter;
        leftLangCombo.get_active_iter (out iter);
        langStore.get_value(iter, 1, out val);
        return (string)val;
    }
    
    private string getRightId() {
        Value val;
        Gtk.TreeIter iter;
        rightLangCombo.get_active_iter (out iter);
        langStore.get_value(iter, 1, out val);
        return (string)val;
    }
    
    private void onLangChange(bool isRight) {
        var leftId = getLeftId();
        var rightId = getRightId();        
        
        var needUpdate = true;                    
        
        if (leftId == rightId) {
            needUpdate = false;
            if (isRight) {
                leftLang = rightLang;
                leftLangCombo.set_active_id(rightLang);
            } else {
                rightLang = leftLang;
                rightLangCombo.set_active_id(leftLang);
            }
            
            topText.buffer.text = bottomText.buffer.text;           
        }                
        
        if (needUpdate) {
            leftLang = leftId;
            rightLang = rightId; 
            onUpdate();   
        }
    }
    
    private void onLeftComboChange() {                    
        onLangChange(false);
    }
    
    private void onRightComboChange() {        
        onLangChange(true);
    }
    
    private void onSwap() {
        var id = getLeftId();
        rightLangCombo.set_active_id(id);
    }
    
    private void onUpdate() {
        if (topText.buffer.text.length < 1) {
            bottomText.buffer.text = "";
            return;
        }
        if ((leftLang == null) || (rightLang == null)) return;                
        service.translate(leftLang, rightLang, topText.buffer.text);
    }
    
    private void onTranslate(string[] text) {
        if ((text == null) || (text.length < 1)) return;
        bottomText.buffer.text = string.joinv("", text);
    }
}